@@ -1,12 +1,12 @@
+# Copyright (c) OpenMMLab. All rights reserved.
 import torch.nn as nn
 import torch.nn.functional as F
 
-from ..weight_init import xavier_init
+from ..utils import xavier_init
 from .registry import UPSAMPLE_LAYERS
 
 UPSAMPLE_LAYERS.register_module('nearest', module=nn.Upsample)
 UPSAMPLE_LAYERS.register_module('bilinear', module=nn.Upsample)
-UPSAMPLE_LAYERS.register_module('deconv', module=nn.ConvTranspose2d)
 
 
 @UPSAMPLE_LAYERS.register_module(name='pixel_shuffle')
@@ -52,14 +52,15 @@ def build_upsample_layer(cfg, *args, **kwargs):
 
     Args:
         cfg (dict): The upsample layer config, which should contain:
+
             - type (str): Layer type.
             - scale_factor (int): Upsample ratio, which is not applicable to
-                deconv.
+              deconv.
             - layer args: Args needed to instantiate a upsample layer.
-        args (argument list): Arguments passed to the `__init__`
-            method of the corresponding conv layer.
-        kwargs (keyword arguments): Keyword arguments passed to the `__init__`
+        args (argument list): Arguments passed to the ``__init__``
             method of the corresponding conv layer.
+        kwargs (keyword arguments): Keyword arguments passed to the
+            ``__init__`` method of the corresponding conv layer.
 
     Returns:
         nn.Module: Created upsample layer.
