@@ -1,4 +1,4 @@
-# Copyright (c) Open-MMLab. All rights reserved.
+# Copyright (c) OpenMMLab. All rights reserved.
 from ...dist_utils import master_only
 from ..hook import HOOKS
 from .base import LoggerHook
@@ -6,6 +6,30 @@ from .base import LoggerHook
 
 @HOOKS.register_module()
 class MlflowLoggerHook(LoggerHook):
+    """Class to log metrics and (optionally) a trained model to MLflow.
+
+    It requires `MLflow`_ to be installed.
+
+    Args:
+        exp_name (str, optional): Name of the experiment to be used.
+            Default None. If not None, set the active experiment.
+            If experiment does not exist, an experiment with provided name
+            will be created.
+        tags (Dict[str], optional): Tags for the current run.
+            Default None. If not None, set tags for the current run.
+        log_model (bool, optional): Whether to log an MLflow artifact.
+            Default True. If True, log runner.model as an MLflow artifact
+            for the current run.
+        interval (int): Logging interval (every k iterations). Default: 10.
+        ignore_last (bool): Ignore the log of last iterations in each epoch
+            if less than `interval`. Default: True.
+        reset_flag (bool): Whether to clear the output buffer after logging.
+            Default: False.
+        by_epoch (bool): Whether EpochBasedRunner is used. Default: True.
+
+    .. _MLflow:
+        https://www.mlflow.org/docs/latest/index.html
+    """
 
     def __init__(self,
                  exp_name=None,
@@ -15,32 +39,6 @@ class MlflowLoggerHook(LoggerHook):
                  ignore_last=True,
                  reset_flag=False,
                  by_epoch=True):
-        """Class to log metrics and (optionally) a trained model to MLflow.
-
-        It requires `MLflow`_ to be installed.
-
-        Args:
-            exp_name (str, optional): Name of the experiment to be used.
-                Default None.
-                If not None, set the active experiment.
-                If experiment does not exist, an experiment with provided name
-                will be created.
-            tags (dict of str: str, optional): Tags for the current run.
-                Default None.
-                If not None, set tags for the current run.
-            log_model (bool, optional): Wheter to log an MLflow artifact.
-                Default True.
-                If True, log runner.model as an MLflow artifact
-                for the current run.
-            interval (int): Logging interval (every k iterations).
-            ignore_last (bool): Ignore the log of last iterations in each epoch
-                if less than `interval`.
-            reset_flag (bool): Whether to clear the output buffer after logging
-            by_epoch (bool): Whether EpochBasedRunner is used.
-
-        .. _MLflow:
-            https://www.mlflow.org/docs/latest/index.html
-        """
         super(MlflowLoggerHook, self).__init__(interval, ignore_last,
                                                reset_flag, by_epoch)
         self.import_mlflow()
